;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x895f71c747fc8461a157ec69f6162cf9 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x85537948a7cd61ee78ff3eb425942280 . "./code.cg2js/cg-printing.fasl")
(#xa9957b055cbe57ad3044d6b322d98057 . "./code.cg2js/cg-caret.fasl")
(#x26edb9b0804089b51b85bcc133f01fee . "./code.cg2js/cg-item-list.fasl")
(#x8ac03186241cba8c15b226c874a8939c . "./code.cg2js/cg-progress-indicator.fasl")
(#xeef3dd0a56dc3deaa8d07af5a566a819 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x8e06aa097c5e42b91fe196354b718ffc . "./code.cg2js/cg-outline.fasl")
(#x011b6c3ee12ae2815b7b80e07e281c04 . "./code.cg2js/cg-palette.fasl")
(#x814adf9798c80e806c1bafc24efa4d6f . "./code.cg2js/cg-selected-object.fasl")
(#xd8fa4a24af0341ed7fdc4c4960ee94b2 . "./code.cg2js/cg-status-bar.fasl")
(#xe14e101f72c025009eb1314bec1764c0 . "./code.cg2js/cg-wrap-string.fasl")
(#xde471d7422954b6c4ce22fc884028249 . "./code.cg2js/cg-fill-texture.fasl")
(#x975bdf67b6b26d1f8ae7c991c67f31b1 . "./code.cg2js/cg-layout.fasl")
(#xfb57d756e5403645cd8c07ddd3c07b5b . "./code.cg2js/cg-timer.fasl")
(#xe28c63d230c2f76e12ccedf6a7d84f8a . "./code.cg2js/cg-dialog-item.fasl")
(#x043093e8b5919471b4daffe31926601e . "./code.cg2js/cg-button.fasl")
(#x7926b7b06ec7e0b75e7c670efaaeb5f0 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x40785072d2b4ecc780f78f911dedb7ba . "./code.cg2js/cg-tray.fasl")
(#xd71b656e7d8972e23f2b644b3f0f2923 . "./code.cg2js/cg-plot-widget.fasl")
(#x80236eb5f11962a7ea38e7a942bd24e7 . "./code.cg2js/cg-pixmap.fasl")
(#x451ffe96c1fade19941eb864dd6a9727 . "./code.cg2js/cg-font-dialog.fasl")
(#x87c3b4cf7770f2468f489a5f2847219f . "./code.cg2js/cg-os-widget.fasl")
(#x1369d229a8e6ce67ac83bdb3fdc7ec6b . "./code.cg2js/cg-text-or-combo.fasl")
(#xb603a4452c778a88dbe4f5d6a9fe7369 . "./code.cg2js/cg-trackbar.fasl")
(#x45e871848c570b89eb118d3c9a79f214 . "./code.cg2js/cg-up-down-control.fasl")
(#x0a108f6d776ddfa651029697be28da41 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x6eba9d2dfeb8297d1791c0e1aced2acc . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x9166f3154cd88f02bcf677405e2e6d08 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x50db51fd554f767fb9c3b0a22942ed8c . "./code.cg2js/cg-bitmap-stream.fasl")
(#xba480e047bf51aea9c836396d69bd246 . "./code.cg2js/cg-chart-widget.fasl")
(#xff5fadd928c344c8d7460d489d818b97 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x0ac611c5e75c1e03662b3059a6c57604 . "./code.cg2js/cg-group-box.fasl")
(#xc373a6fb52e244ff285df0bbe8c32f00 . "./code.cg2js/cg-static-text.fasl")
(#xc9beca632fbb279c1b4456aef1402a75 . "./code.cg2js/cg-class-slot-grid.fasl")
(#xc7f3a57e91d070d4ce3f6655c438c9e8 . "./code.cg2js/cg-color-dialog.fasl")
(#x98be5deec1aafe45501c4f6cd149697a . "./code.cg2js/cg-intersect.fasl")
(#xe98c2a4e2ff337e806eed44ff97c0278 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x912ff599ae9ac0786b1f38240f646e8e . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x20487f878f7ec957d0f938e755b1643e . "./code.cg2js/cg-html-dialog.fasl")
(#x3678763861fa80079c396f7a8409ffd9 . "./code.cg2js/cg-lettered-menu.fasl")
(#xeae22431095fcbefb2444b2ad13e7e0b . "./code.cg2js/cg-class-grid.fasl")
(#x0274c588f6c7d9a57e06ab38153e6ffe . "./code.cg2js/cg-pixmap-printing.fasl")
(#xb9ca6a528461eba374c282e83be83784 . "./code.cg2js/cg-calendar.fasl")
(#x6e4e11288b3d5bb8347c3e46cf0de3c7 . "./code.cg2js/cg-editable-text.fasl")
(#xed28069e65e0661b3a6ac833b4e77f39 . "./code.cg2js/cg-property.fasl")
(#xfb66abd954932fe7ec047e3c8678a19d . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x0e43c7da8865923c27eb961a72900530 . "./code.cg2js/cg-grid-widget.fasl")
(#xd386292a45f7624e98e4721b5b66597a . "./code.cg2js/cg-paren-matching.fasl")
(#x7aa0a5c95214fa6d7d0e617d3a84ace4 . "./code.cg2js/cg-dev.fasl")
(#x04be360908cc0ef0a760a6f8ddaf483e . "./code.cg2js/cg-drawable.fasl")
(#xc7188b52edd8b6f76186ce5e09ec9a68 . "./code.cg2js/cg-curve.fasl")
(#x8601c4d6eedaf409d6dbdb26a754bc0b . "./code.cg2js/cg-lisp-widget.fasl")
(#xbfdcd196ea1e24fb3cda8a9cfecc8fee . "./code.cg2js/cg-dde-utils.fasl")
(#xcd6481b04ef45a60f15f63874b1b1cd3 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x139d91960e2d7df09401fbdb77fc0227 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x99e3e6aa871ceb2b234bf3fdf3145ba6 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x81a95d43ec98e365701bdc13acb757ee . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x448713f9b039131c7bf8921270e0b747 . "./code.cg2js/cg-lamp.fasl")
(#x8970a904419426f11efa904dc7b39224 . "./code.cg2js/cg-common-control.fasl")
(#x8c23b7ba2b3fc9038cecc8cde91b1dd1 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xd470eff86559629d0434a2873d78aff5 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x5a1fdbd08e966ae95329c2276b5eefad . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x3073c7c9b74c50a0247c4528bef5d2f0 . "./code.cg2js/cg-project-window.fasl")
(#x4893ca289f8fdb6b28c5366806d9676d . "./code.cg2js/cg-sample-file-menu.fasl")
(#x6b694b8cead499543d121904f1275804 . "./code.cg2js/cg-class-support.fasl")
(#xb8433ea90a554799c7d143ab8917e210 . "./code.cg2js/cg-multi-picture-button.fasl")
(#xbbe90a58ff960bde90f1741a4e5d1ac6 . "./code.cg2js/cg-yes-no-list.fasl")
(#x1c933841cfc3f2ec6b9fccdc412580f0 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xebcddf3993c9019545f0d95114301da0 . "./code.cg2js/cg-string-dialog.fasl")
(#xb59e0cbdd7df2a6587993d66183e72bc . "./code.cg2js/cg-menu-tooltip.fasl")
(#x764825c7600f840e9a631a758241f046 . "./code.cg2js/cg-dropping-outline.fasl")
(#xf94ec71fe43c28315a93e6fef6ebfa1c . "./code.cg2js/cg.fasl")
(#xb220b0a012818e074c57e94bdb5ced93 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x8b4557a32145721cc44cecdda4e7181e . "./code.cg2js/cg-scaling-stream.fasl")
(#xcfdd709de7810a44cece0795e74980b0 . "./code.cg2js/cg-yes-no-string.fasl")
(#x482d5a92ed3ae08c15f2b192e317289c . "./code.cg2js/cg-icon-pixmap.fasl")
(#xfe9a9f6e6aaa4d4049c1ad0b1e4f5a02 . "./code.cg2js/cg-web-browser.fasl")
(#x2f38c792b59a6944fca6178db41f951b . "./code.cg2js/cg-clipboard-stack.fasl")
(#x8700841007b6608e41603e7c4b7d844d . "./code.cg2js/cg-drag-and-drop.fasl")
(#xb0cd5d63d006876b03b26538be4d8016 . "./code.cg2js/cg-message-dialog.fasl")
(#xcfc792a364778a08e2ab6991b7b0c1cf . "./code.cg2js/cg-comtab.fasl")
(#x52193bedc0c4e8dcf22f6692186e59a9 . "./code.cg2js/cg-base.fasl")
(#x390e547120ae798cebcf2d951d080cc4 . "./code.cg2js/cg-pixmap-widget.fasl")
(#xbfacbaed0e5789222e94e467eb5a92e9 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x77127317f8bb2c3e3e4c0a00cbdfbc8d . "./code.cg2js/cg-utility-dialog.fasl")
(#x57edbf5c9deef7e3597c247c9d64a65a . "./code.cg2js/cg-object-editor.fasl")
(#xf300e06e57e142b981a7bccb1f2ab7d1 . "./code.cg2js/cg-edit-in-place.fasl")
(#x0bc039def56d619b8ce56b9f6794d2da . "./code.cg2js/cg-radio-button.fasl")
(#x89e6b77cbe25e4f3619afd320edd461d . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xee051805244baeb1e47d06e19b8ed52b . "./code.cg2js/cg-nodes.fasl")
(#x95663d18fc59c407576f98ff57af0e1b . "./code.cg2js/cg-get-position.fasl")
(#xd9bca1c8374e117796b296fac7aee4f5 . "./code.cg2js/cg-directory-dialog.fasl")
(#x44983d172d1cf93ebcc3ea1cc7d93684 . "./code.cg2js/cg-lisp-text.fasl")
(#x00af56c1a740092cd81e4e393a68583e . "./code.cg2js/cg-combo-box.fasl")
(#x9d757b22070886656b292b6deb4af1dd . "./code.cg2js/cg-text-widget.fasl")
(#xc9cd6c06b61ee20a4a514f1115c39fe3 . "./code.cg2js/cg-list-view.fasl")
(#x115944064c90c5624b27c2fca0045047 . "./code.cg2js/cg-graphics-context.fasl")
(#x77c33394e5c25145bc7e906f37a8a803 . "./code.cg2js/cg-get-pixmap.fasl")
(#xb6e7b833ec57cdffcd61113208612d30 . "./code.cg2js/cg-menu.fasl")
(#xbd548c01a1fe5190dd00e86ec91f68d9 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xe55624996df1673517ee6a25921bc8a5 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xbfa00a3fc0a40c6658fcbc411b04a403 . "./code.cg2js/cg-tooltip.fasl")
(#x75e706c7208818a3b1f446b2c25f25a4 . "./code.cg2js/cg-html-widget.fasl")
(#x8bd6ef9c37a5ef5192f521418072780a . "./code.cg2js/cg-choice-list.fasl")
(#xfb3e2786715c27ab2bcbe7b657c711f4 . "./code.cg2js/cg-icon.fasl")
(#x7ba73a24bc3b8eb3e6ca6c2c20ea641f . "./code.cg2js/cg-acache.fasl")
(#x5ee5ebb8a609c16a3710d3541fb5c717 . "./code.cg2js/cg-template-string.fasl")
(#xb610a69896858a5de055acf4fc6e9457 . "./code.cg2js/cg-bitmap-pane.fasl")
(#xcc735e78ab2a31f0fb8be10a8777a5d2 . "./code.cg2js/cg-toggling-widget.fasl")
(#x9fe4fe3b47d4036619c0433ba96dbbac . "./code.cg2js/cg-tab-control.fasl")
(#x148254db831d558f464a1ea6aba46c3d . "./code.cg2js/cg-split-bar.fasl")
(#x7a472d3c17b4f497d42329d6f054d735 . "./code.cg2js/cg-picture-widget.fasl")
(#xa1fd989f7e3bf4144a1e21bfd1063352 . "./code.cg2js/cg-player.fasl")
(#xe3e573c4aa47e79a33dbb7412e7ecc5e . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x69fba270dc6be61524c1d9030c3be0dc . "./code.cg2js/cg-choose-printer.fasl")
(#x9e510630c3eb30f4aa9b5ee905677e75 . "./code.cg2js/cg-os-window.fasl")
(#x3a6c07eb5176dc80111ec2374b512fb6 . "./code.cg2js/cg-scroll-bar.fasl")
(#x2cc408e285b5a24914e31c3cbd97795f . "./code.cg2js/cg-clipboard.fasl")
(#x0c7fb16db336a5a647df53c38e8427d8 . "./code.cg2js/cg-find-string-dialog.fasl")
(#xa626b8d11ab3eda7a753440ffccf3b8c . "./code.cg2js/cg-hotspot.fasl")
(#xbeb86c15397efd1399e06e3d938c30ff . "./code.cg2js/cg-check-box.fasl")
(#x61e0b680adc7c0f0898b8c4587dff822 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x2d617192ebc7b1f3b9645eaa4f37a8bb . "./code.cg2js/cg-file-dialog.fasl")
(#x991d85d4cdd8be10c42f647871666e87 . "./code.cg2js/cg-toolbar.fasl")
(#x5d3a35663306bc73d656c17196a0e140 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x9a0117ccc51065608feb03e8f97963e9 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x2944931d9cc43aa76c304e8ac19e9840 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x314f1a4d5df40529a9913ea137a90619 . "./code.idejs/ide-project.fasl")
(#x8212f251459f0068fadae28b7f8f8c29 . "./code.idejs/ide-intro.fasl")
(#x461d90db0861a20dd4db933305b0c694 . "./code.idejs/ide-find-in-files.fasl")
(#xa0d4d66c390defa0230f8509c1f3309f . "./code.idejs/ide-grapher.fasl")
(#xaa9b4efdb171478516c5b0b786050a9a . "./code.idejs/ide-inspector.fasl")
(#x44a5e3a00eac4efc7b3a3c462c150a73 . "./code.idejs/ide-editor.fasl")
(#x989b562cd75b135c7b54b62cdd07e1b9 . "./code.idejs/ide-window-list.fasl")
(#x977694fa3591bea8f42537143ddb89ac . "./code.idejs/ide-process-browser.fasl")
(#xb892655dde8d8572fd1bc11a4fe646a4 . "./code.idejs/ide-patch-dialog.fasl")
(#x51805b0b23161ce67fca0d26279ce448 . "./code.idejs/ide-class-grapher.fasl")
(#x049a8ee8710aa8a4a736667ee452633e . "./code.idejs/ide-base.fasl")
(#x660c6396899acff30ca3394d805eb228 . "./code.idejs/ide.fasl")
(#x8472c3508ba08c79daaa2ae9013918e3 . "./code.idejs/ide-trace.fasl")
(#xcccbdf85b122c687a93e5aaca86a551d . "./code.idejs/ide-form.fasl")
(#xfb3cca1e88a6cd081df07cdd97026017 . "./code.idejs/ide-find-definitions.fasl")
(#xd31082ef65182bfa7af480ac33f53423 . "./code.idejs/ide-apropos.fasl")
(#x4ff3fbfd01e049cc9ddbe149ba1051e8 . "./code.idejs/ide-class-browser.fasl")
(#x0a4664e70e4461d957acf6c32122441d . "./code.idejs/ide-navigator.fasl")
(#xb0c8cb679118cc61e51aaad93b1e2d77 . "./code.idejs/ide-clipboard.fasl")
(#xae16fab949d90ce89f29c8e3c8fb1e89 . "./code.idejs/ide-acache.fasl")
(#xf2ddf28d536b28cc54c94c84d92366ac . "./code.idejs/ide-keyshort.fasl")
(#xe1936f783f3b7f4785f5045d201405cd . "./code.idejs/ide-package-list.fasl")
(#x4dc80f83d60cf0714be345b1174a5599 . "./code.idejs/ide-dev.fasl")
(#x29c4a4cb691acde2a2fe4116aa720ac8 . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
